namespace eval myrun {
    variable tdynexe
    variable ramsexe
    variable tdyndat
    variable ramsdat
    variable tcldll
    variable dir_ptype
    variable exetype
    variable restype
}

proc myrun::get_sysinfo {} {
    variable tdynexe
    variable dir_ptype

    if { ![info exists tdynexe] } { myrun::setexe $dir_ptype }

    set remotedir [send create_remote_new_dir tdyn]
    copy_file_to_server -binary 1 $tdynexe $remotedir
    set data [send exec_in_dir $remotedir [file tail $tdynexe] -outputinfo]
    send file delete -force $remotedir
    return $data
}

# execution types (exetype): 
#       2D will execute tdyn2d
#       2DA will execute tdyn2da
#       3D will execute tdyn3d
proc myrun::run_tdyn { gid_model password_file } {
    variable tdynexe
    variable ramsexe
    variable tdyndat
    variable ramsdat
    variable tcldll
    variable dir_ptype
    variable exetype
    variable restype
    
    if { ![info exists tdynexe] } { myrun::setexe $dir_ptype }
    
    set l_base [file join $gid_model [file tail [file root $gid_model]]]

    if { ![catch { set filelist [glob "$l_base.flavia*"] }] } {
	foreach i $filelist {
	    catch { file delete -force $i }        
	}
    }
    set remotedir [send create_remote_new_dir tdyn]

    # Sending exe and dat files to the remote server 
    file rename -force $l_base.dat $l_base.flavia
    set tdyndat $l_base.flavia
    set tdynout [file tail $l_base.flavia.out]
    set tdynerr [file tail $l_base.flavia.err]
    set infoext flavia.inf
    foreach "file t" [list $tdynexe binary $tcldll binary \
	    $password_file ascii $tdyndat ascii] {
	switch $t binary { set b 1 } ascii { set b 0 }
	copy_file_to_server -binary $b $file $remotedir
    }
    # Sending data directory to the remote server
    if { [file exists [file join $gid_model data]] } {
	send file mkdir [file join $remotedir data]
	if { ![catch { set filelist [glob -dir [file join $gid_model data] *] }] } {
	    foreach "file" $filelist {
		copy_file_to_server -binary 0 $file [file join $remotedir data]
	    }
	}
    }
    # Sendind password file
    if { [file tail $password_file] ne "password.txt" } {
	send file rename [file join $remotedir [file tail $password_file]] \
		[file join $remotedir password.txt]
    }

    # Remote execution of the program/s
    switch $exetype {
	2D {  
	    set pidt [compass_client::exec_in_dir $remotedir [file tail $tdynexe] -name \
		    [file tail $tdyndat] -2d  > $tdynout &]
	}
	2DA {
	    set pidt [compass_client::exec_in_dir $remotedir [file tail $tdynexe] -name \
		    [file tail $tdyndat] -2da  > $tdynout &]
	}
	3D {
	    set pidt [compass_client::exec_in_dir $remotedir [file tail $tdynexe] -name \
		    [file tail $tdyndat] -3d  > $tdynout &]
	}
	default {
	    set fout [open [file join $gid_model $l_base.flavia.err] a]
	    puts $fout "Wrong execution data for Tdyn found in myrun::run_tdyn"
	    close $fout
	    return
	}
    }

    # Checking execution process
    set r_base [file join $remotedir [file tail [file root $gid_model]]]

    after 1000
    set mtime ""
    set RunningProcess running
    while 1 {
	if { [send file exists $r_base.$infoext] } {
	    set new_mtime [send file mtime $r_base.$infoext]
	    if { $mtime eq "" || $new_mtime > $mtime } {
		copy_file_from_server $r_base.$infoext $gid_model
	    }
	    set mtime $new_mtime
	}
	# Is Tdyn still alive?
	if { ![send process_exists $pidt] } { break }
	
	after 5000 [list set waiting_var 1]
	vwait ::waiting_var
    }
    unset pidt
    unset tdynexe

    foreach i [send glob -dir $remotedir [file tail [file root $gid_model]]*] {
	if { [info exists tdyndat] && ([file tail $i] eq [file tail $tdyndat.res]) && ($restype ne "Ascii") } {
	    copy_file_from_server -nocomplain 1 -binary 1 $i $gid_model
	} else {
	    copy_file_from_server -nocomplain 1 -binary 0 $i $gid_model
	}
    }
    
    send file delete -force $remotedir
    unset remotedir
}

# execution types (exetype): 
#       RAM will execute ramsolid
proc myrun::run_rams { gid_model password_file } {
    variable tdynexe
    variable ramsexe
    variable tdyndat
    variable ramsdat
    variable tcldll
    variable dir_ptype
    variable exetype
    variable restype
    
    if { ![info exists ramsexe] } { myrun::setexe $dir_ptype }
    
    set l_base [file join $gid_model [file tail [file root $gid_model]]]

    if { ![catch { set filelist [glob "$l_base.flavia*"] }] } {
	foreach i $filelist {
	    catch { file delete -force $i }
	}
    }
    if { ![catch { set filelist [glob "$l_base-ramsolid*"] }] } {
	foreach i $filelist {
	    catch { file delete -force $i }
	}
    }
    set remotedir [send create_remote_new_dir tdyn]

    # Sending exe and dat files to the remote server 
    file rename $l_base-1.dat $l_base-ramsolid.dat
    set ramsdat $l_base-ramsolid.dat
    set ramsinf [file tail $l_base-ramsolid.info]
    set ramserr [file tail $l_base-ramsolid.err]
    set ramswar [file tail $l_base-ramsolid.war]
    set infoext info
    foreach "file t" [list $ramsexe binary $tcldll binary \
	    $password_file ascii $ramsdat ascii] {
	switch $t binary { set b 1 } ascii { set b 0 }
	copy_file_to_server -binary $b $file $remotedir
    }
    if { [file tail $password_file] ne "password.txt" } {
	send file rename [file join $remotedir [file tail $password_file]] \
		[file join $remotedir password.txt]
    }

    # Remote execution of the program/s
    set pidr [compass_client::exec_in_dir $remotedir [file tail $ramsexe] -i [file tail $ramsinf] \
	    -e [file tail $ramserr] -w [file tail $ramswar] -a [file tail $ramsdat] &]
    
    # Checking execution process
    set r_base [file join $remotedir [file tail [file root $gid_model]]]

    after 1000
    set mtime ""
    set RunningProcess running
    while 1 {
	if { [send file exists $r_base.$infoext] } {
	    set new_mtime [send file mtime $r_base.$infoext]
	    if { $mtime eq "" || $new_mtime > $mtime } {
		copy_file_from_server $r_base.$infoext $gid_model
	    }
	    set mtime $new_mtime
	}
	# Is Ramsolid still alive
	if { ![send process_exists $pidr] } { break }

	after 5000 [list set waiting_var 1]
	vwait ::waiting_var
    }
    unset pidr
    unset ramsexe

    foreach i [send glob -dir $remotedir [file tail [file root $gid_model]]*] {
	if { [info exists tdyndat] && ([file tail $i] eq [file tail $tdyndat.res]) && ($restype ne "Ascii") } {
	    copy_file_from_server -nocomplain 1 -binary 1 $i $gid_model
	} else {
	    copy_file_from_server -nocomplain 1 -binary 0 $i $gid_model
	}
    }
    catch { file rename -force $l_base-ramsolid.flavia.res $l_base.flavia.res }

    send file delete -force $remotedir
    unset remotedir
}

# execution types (exetype): 
#       3DRAM will execute tdyn and ramsolid
proc myrun::run_ramsandtdyn { gid_model password_file } {
    variable tdynexe
    variable ramsexe
    variable tdyndat
    variable ramsdat
    variable tcldll
    variable dir_ptype
    variable exetype
    variable restype
    
    if { ![info exists tdynexe] || ![info exists ramsexe] } { myrun::setexe $dir_ptype }
    
    set l_base [file join $gid_model [file tail [file root $gid_model]]]

    if { ![catch { set filelist [glob "$l_base.flavia*"] }] } {
	foreach i $filelist {
	    catch { file delete -force $i }
	}
    }
    if { ![catch { set filelist [glob "$l_base-ramsolid*"] }] } {
	foreach i $filelist {
	    catch { file delete -force $i }
	}
    }
    set remotedir [send create_remote_new_dir tdyn]

    # Sending exe and dat files to the remote server 
    file rename $l_base.dat $l_base.flavia
    file rename $l_base-1.dat $l_base-ramsolid.dat
    set tdyndat $l_base.flavia
    set tdynout [file tail $l_base.flavia.out]
    set tdynerr [file tail $l_base.flavia.err]
    set ramsdat $l_base-ramsolid.dat
    set ramsinf [file tail $l_base-ramsolid.info]
    set ramserr [file tail $l_base-ramsolid.err]
    set ramswar [file tail $l_base-ramsolid.war]
    set infoext flavia.inf
    foreach "file t" [list $tdynexe binary $ramsexe binary $tcldll binary \
	    $password_file ascii $tdyndat ascii $ramsdat ascii] {
	switch $t binary { set b 1 } ascii { set b 0 }
	copy_file_to_server -binary $b $file $remotedir
    } 
    # Sending data directory to the remote server
    if { [file exists [file join $gid_model data]] } {
	send file mkdir [file join $remotedir data]
	if { ![catch { set filelist [glob -dir [file join $gid_model data] *] }] } {
	    foreach "file" $filelist {
		copy_file_to_server -binary 0 $file [file join $remotedir data]
	    }
	}
    }
    # Sendind password file
    if { [file tail $password_file] ne "password.txt" } {
	send file rename [file join $remotedir [file tail $password_file]] \
		[file join $remotedir password.txt]
    }
	
    # Remote execution of the program/s
    for { set port 3256 } { $port <= 13256 } { incr port } {
	set err [catch { send socket -server "" $port } sock]
	if { !$err } {
	    send close $sock
	    break
	}
    }
    if { $port > 13256 } {
	set fout [open [file join $dir $l_base.flavia.err] a]
	puts $fout "Failed to open TCP-IP port in the range 3256-13256."
	puts $fout "Contact your system administrator and check configuration of firewalls and anti-virus"
	close $fout
	return
    }
    set pidt [compass_client::exec_in_dir $remotedir [file tail $tdynexe] -name \
	    [file tail $tdyndat] -3d -coupling $port 10000 > [file tail $tdynout] &]
    set pidr [compass_client::exec_in_dir $remotedir [file tail $ramsexe] -tdyncoupling $port 10000 -i [file tail $ramsinf] \
	    -e [file tail $ramserr] -w [file tail $ramswar] -a [file tail $ramsdat] &]

    # Checking execution process
    set r_base [file join $remotedir [file tail [file root $gid_model]]]

    after 1000
    set mtime ""
    set RunningProcess running
    set ErrorInProcess "no"
    while 1 {
	if { [send file exists $r_base.$infoext] } {
	    set new_mtime [send file mtime $r_base.$infoext]
	    if { $mtime eq "" || $new_mtime > $mtime } {
		copy_file_from_server $r_base.$infoext $gid_model
	    }
	    set mtime $new_mtime
	}

	# Is any process still alive
	if { ![send process_exists $pidt] & ![send process_exists $pidt] } { break }
	# Let's check if Tdyn is running
	if { ![send process_exists $pidt] & [send file exists [join $remotedir $tdynerr]] && \
		[send file size [join $remotedir $tdynerr]] > 0 } {
	    send end_process -force 1 $pidr
	    set ErrorInProcess "yes"
	    break
	}
	# Let's check if Ramseries is running
	if { ![send process_exists $pidr] & [send file exists [join $remotedir $ramserr]] && \
		[send file size [join $remotedir $ramserr]] > 0 } {
	    send end_process -force 1 $pidt
	    set ErrorInProcess "yes"
	    break
	}
	
	after 5000 [list set waiting_var 1]
	vwait ::waiting_var
    }
    unset pidr
    unset pidt
    unset tdynexe
    unset ramsexe

    foreach i [send glob -dir $remotedir [file tail [file root $gid_model]]*] {
	if { [info exists tdyndat] && ([file tail $i] eq [file tail $tdyndat.res]) && ($restype ne "Ascii") } {
	    copy_file_from_server -nocomplain 1 -binary 1 $i $gid_model
	} else {
	    copy_file_from_server -nocomplain 1 -binary 0 $i $gid_model
	}
    }
  
    if { $ErrorInProcess=="no" } {
	if { ![file exists $l_base.flavia.res] || \
	    [file size $l_base.flavia.res] == 0 } {
	    set fout [open $l_base.flavia.err a]
	    puts $fout "Tdyn failed. Results are not available."
	    close $fout
	}
	if { ![file exists $l_base-ramsolid.flavia.res]  || \
	    [file size $l_base-ramsolid.flavia.res] == 0 } {
	    set fout [open $l_base.flavia.err a]
	    puts $fout "Ramsolid failed. Results are not available."
	    close $fout
	}
    }
    
    foreach i [list .err .war] {
	set f $l_base-ramsolid$i
	if { [file exists $f] && [file size $f] > 0 } {
	    set fin [open $f r]
	    set fout [open $l_base.flavia$i a]
	    puts -nonewline $fout [read $fin]
	    close $fin
	    close $fout
	}
    }
    set ftdyn $l_base.flavia.res
    set fram $l_base-ramsolid.flavia.res
    if { [file exists $ftdyn] && [file size $ftdyn] > 0 && \
	     [file exists $fram] && [file size $fram] > 0 } {
	set fin [open $fram r]
	gets $fin
	set fout [open $ftdyn a]
	puts -nonewline $fout [read $fin]
	close $fin
	close $fout
	file delete $fram
    }

    send file delete -force $remotedir
    unset remotedir
}

proc myrun::finish { basename dir } {

    set flavia [file join $dir $basename.flavia.res]
    if { [file exists $flavia] && [file size $flavia] > 0 } {
	# file delete [file join $dir $basename.flavia]
    } else {
	set fout [open [file join $dir $basename.flavia.err] a]
	puts $fout "Execution failed."
	close $fout
    }
}

proc myrun::setexe { dir } {
    global tcl_platform
    variable tdynexe
    variable ramsexe
    variable tdyndat
    variable ramsdat
    variable tcldll
    variable dir_ptype

    switch $::tcl_platform(platform) {
	"windows" { 
	    set tdynexe [file join $dir_ptype .. scripts tdyn.exe]
	    set ramsexe [file join $dir_ptype .. scripts ramsolid.exe]
	    set tcldll [file join $dir_ptype .. scripts tcl84.dll]
	}
	"default" { 
	    set tdynexe [file join $dir_ptype .. scripts tdyn]
	    set ramsexe [file join $dir_ptype .. scripts ramsolid]
	    set tcldll [file join $dir_ptype .. scripts tcl84.dll]
	}
    }
}















